@@ -2,7 +2,7 @@
  *
  *  Connection Manager
  *
- *  Copyright (C) 2007-2010  Intel Corporation. All rights reserved.
+ *  Copyright (C) 2007-2012  Intel Corporation. All rights reserved.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License version 2 as
@@ -53,6 +53,11 @@ enum connman_wispr_result {
 	CONNMAN_WISPR_RESULT_FAILED  = 3,
 };
 
+struct wispr_route {
+	char *address;
+	int if_index;
+};
+
 struct connman_wispr_portal_context {
 	struct connman_service *service;
 	enum connman_ipconfig_type type;
@@ -75,6 +80,8 @@ struct connman_wispr_portal_context {
 	char *wispr_formdata;
 
 	enum connman_wispr_result wispr_result;
+
+	GSList *route_list;
 };
 
 struct connman_wispr_portal {
@@ -115,9 +122,39 @@ static void connman_wispr_message_init(struct connman_wispr_message *msg)
 	msg->location_name = NULL;
 }
 
+static void free_wispr_routes(struct connman_wispr_portal_context *wp_context)
+{
+	while (wp_context->route_list != NULL) {
+		struct wispr_route *route = wp_context->route_list->data;
+
+		DBG("free route to %s if %d type %d", route->address,
+				route->if_index, wp_context->type);
+
+		switch(wp_context->type) {
+		case CONNMAN_IPCONFIG_TYPE_IPV4:
+			connman_inet_del_host_route(route->if_index,
+					route->address);
+			break;
+		case CONNMAN_IPCONFIG_TYPE_IPV6:
+			connman_inet_del_ipv6_host_route(route->if_index,
+					route->address);
+			break;
+		case CONNMAN_IPCONFIG_TYPE_UNKNOWN:
+			break;
+		}
+
+		g_free(route->address);
+		g_free(route);
+
+		wp_context->route_list =
+			g_slist_delete_link(wp_context->route_list,
+					wp_context->route_list);
+	}
+}
+
 static void free_connman_wispr_portal_context(struct connman_wispr_portal_context *wp_context)
 {
-	DBG("");
+	DBG("context %p", wp_context);
 
 	if (wp_context == NULL)
 		return;
@@ -141,6 +178,8 @@ static void free_connman_wispr_portal_context(struct connman_wispr_portal_contex
 	g_free(wp_context->wispr_password);
 	g_free(wp_context->wispr_formdata);
 
+	free_wispr_routes(wp_context);
+
 	g_free(wp_context);
 }
 
@@ -347,7 +386,7 @@ static void xml_wispr_parser_callback(const char *str, gpointer user_data)
 	result = g_markup_parse_context_parse(parser_context,
 					str, strlen(str), NULL);
 	if (result == TRUE)
-		result = g_markup_parse_context_end_parse(parser_context, NULL);
+		g_markup_parse_context_end_parse(parser_context, NULL);
 
 	g_markup_parse_context_free(parser_context);
 }
@@ -389,13 +428,62 @@ static void portal_manage_status(GWebResult *result,
 						wp_context->type);
 }
 
+static gboolean wispr_route_request(const char *address, int ai_family,
+		int if_index, gpointer user_data)
+{
+	int result = -1;
+	struct connman_wispr_portal_context *wp_context = user_data;
+	const char *gateway;
+	struct wispr_route *route;
+
+	gateway = __connman_ipconfig_get_gateway_from_index(if_index,
+		wp_context->type);
+
+	DBG("address %s if %d gw %s", address, if_index, gateway);
+
+	if (gateway == NULL)
+		return FALSE;
+
+	route = g_try_new0(struct wispr_route, 1);
+	if (route == 0) {
+		DBG("could not create struct");
+		return FALSE;
+	}
+
+	switch(wp_context->type) {
+	case CONNMAN_IPCONFIG_TYPE_IPV4:
+		result = connman_inet_add_host_route(if_index, address,
+				gateway);
+		break;
+	case CONNMAN_IPCONFIG_TYPE_IPV6:
+		result = connman_inet_add_ipv6_host_route(if_index, address,
+				gateway);
+		break;
+	case CONNMAN_IPCONFIG_TYPE_UNKNOWN:
+		break;
+	}
+
+	if (result < 0) {
+		g_free(route);
+		return FALSE;
+	}
+
+	route->address = g_strdup(address);
+	route->if_index = if_index;
+	wp_context->route_list = g_slist_prepend(wp_context->route_list, route);
+
+	return TRUE;
+}
+
 static void wispr_portal_request_portal(struct connman_wispr_portal_context *wp_context)
 {
 	DBG("");
 
 	wp_context->request_id = g_web_request_get(wp_context->web,
 					wp_context->status_url,
-					wispr_portal_web_result, wp_context);
+					wispr_portal_web_result,
+					wispr_route_request,
+					wp_context);
 
 	if (wp_context->request_id == 0)
 		wispr_portal_error(wp_context);
@@ -432,17 +520,46 @@ static gboolean wispr_input(const guint8 **data, gsize *length,
 	return FALSE;
 }
 
+static void wispr_portal_browser_reply_cb(struct connman_service *service,
+					connman_bool_t authentication_done,
+					const char *error, void *user_data)
+{
+	struct connman_wispr_portal_context *wp_context = user_data;
+
+	DBG("");
+
+	if (service == NULL || wp_context == NULL)
+		return;
+
+	if (authentication_done == FALSE) {
+		wispr_portal_error(wp_context);
+		free_wispr_routes(wp_context);
+		return;
+	}
+
+	/* Restarting the test */
+	__connman_wispr_start(service, wp_context->type);
+}
+
 static void wispr_portal_request_wispr_login(struct connman_service *service,
 				connman_bool_t success,
 				const char *ssid, int ssid_len,
 				const char *username, const char *password,
 				gboolean wps, const char *wpspin,
-				void *user_data)
+				const char *error, void *user_data)
 {
 	struct connman_wispr_portal_context *wp_context = user_data;
 
 	DBG("");
 
+	if (error != NULL && g_strcmp0(error,
+			"net.connman.Agent.Error.LaunchBrowser") == 0) {
+		__connman_agent_request_browser(service,
+				wispr_portal_browser_reply_cb,
+				wp_context->redirect_url, wp_context);
+		return;
+	}
+
 	g_free(wp_context->wispr_username);
 	wp_context->wispr_username = g_strdup(username);
 
@@ -493,7 +610,7 @@ static gboolean wispr_manage_message(GWebResult *result,
 
 		if (__connman_agent_request_login_input(wp_context->service,
 					wispr_portal_request_wispr_login,
-					wp_context) != -EIO)
+					wp_context) != -EINPROGRESS)
 			wispr_portal_error(wp_context);
 
 		break;
@@ -525,24 +642,6 @@ static gboolean wispr_manage_message(GWebResult *result,
 	return FALSE;
 }
 
-static void wispr_portal_browser_reply_cb(struct connman_service *service,
-					connman_bool_t authentication_done,
-					void *user_data)
-{
-	struct connman_wispr_portal_context *wp_context = user_data;
-
-	if (service == NULL || wp_context == NULL)
-		return;
-
-	if (authentication_done == FALSE) {
-		wispr_portal_error(wp_context);
-		return;
-	}
-
-	/* Restarting the test */
-	__connman_wispr_start(service, wp_context->type);
-}
-
 static gboolean wispr_portal_web_result(GWebResult *result, gpointer user_data)
 {
 	struct connman_wispr_portal_context *wp_context = user_data;
@@ -593,8 +692,9 @@ static gboolean wispr_portal_web_result(GWebResult *result, gpointer user_data)
 
 		break;
 	case 302:
-		if (g_web_result_get_header(result, "Location",
-						&redirect) == FALSE) {
+		if (g_web_supports_tls() == FALSE ||
+				g_web_result_get_header(result, "Location",
+							&redirect) == FALSE) {
 			__connman_agent_request_browser(wp_context->service,
 					wispr_portal_browser_reply_cb,
 					wp_context->status_url, wp_context);
@@ -606,9 +706,11 @@ static gboolean wispr_portal_web_result(GWebResult *result, gpointer user_data)
 		wp_context->redirect_url = g_strdup(redirect);
 
 		wp_context->request_id = g_web_request_get(wp_context->web,
-				redirect, wispr_portal_web_result, wp_context);
+				redirect, wispr_portal_web_result,
+				wispr_route_request, wp_context);
 
 		goto done;
+	case 400:
 	case 404:
 		if (__connman_service_online_check_failed(wp_context->service,
 							wp_context->type) == 0)
@@ -619,6 +721,7 @@ static gboolean wispr_portal_web_result(GWebResult *result, gpointer user_data)
 		break;
 	}
 
+	free_wispr_routes(wp_context);
 	wp_context->request_id = 0;
 done:
 	wp_context->wispr_msg.message_type = -1;
@@ -633,9 +736,6 @@ static void proxy_callback(const char *proxy, void *user_data)
 
 	wp_context->token = 0;
 
-	if (getenv("CONNMAN_WEB_DEBUG"))
-		g_web_set_debug(wp_context->web, web_debug, "WEB");
-
 	if (proxy != NULL && g_strcmp0(proxy, "DIRECT") != 0)
 		g_web_set_proxy(wp_context->web, proxy);
 
@@ -700,22 +800,28 @@ static int wispr_portal_detect(struct connman_wispr_portal_context *wp_context)
 
 	if_index = connman_inet_ifindex(interface);
 	if (if_index < 0) {
+		DBG("Could not get ifindex");
 		err = -EINVAL;
 		goto done;
 	}
 
 	nameservers = connman_service_get_nameservers(wp_context->service);
 	if (nameservers == NULL) {
+		DBG("Could not get nameservers");
 		err = -EINVAL;
 		goto done;
 	}
 
 	wp_context->web = g_web_new(if_index);
 	if (wp_context->web == NULL) {
+		DBG("Could not set up GWeb");
 		err = -ENOMEM;
 		goto done;
 	}
 
+	if (getenv("CONNMAN_WEB_DEBUG"))
+		g_web_set_debug(wp_context->web, web_debug, "WEB");
+
 	if (wp_context->type == CONNMAN_IPCONFIG_TYPE_IPV4) {
 		g_web_set_address_family(wp_context->web, AF_INET);
 		wp_context->status_url = STATUS_URL_IPV4;
